import { Component, EventEmitter, Inject, Input, NgModule, Output, ViewChild } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { Headers, Http, HttpModule, RequestOptions } from '@angular/http';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { MAT_DIALOG_DATA, MatButtonModule, MatCardModule, MatCheckboxModule, MatDatepickerModule, MatDialog, MatDialogModule, MatDialogRef, MatFormFieldModule, MatInputModule, MatMenuModule, MatPaginator, MatPaginatorModule, MatSelectModule, MatSnackBar, MatSnackBarModule, MatSort, MatSortModule, MatTableModule, MatTabsModule } from '@angular/material';
import { DataSource } from '@angular/cdk/collections';
import { Observable as Observable$1 } from 'rxjs/Observable';
import 'rxjs/add/observable/of';
import 'rxjs/add/observable/merge';
import 'rxjs/add/operator/map';
import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';

class ObjectAddDeleteModifyComponent {
    /**
     * @param {?} http
     * @param {?} data
     * @param {?} dialogRef
     */
    constructor(http$$1, data, dialogRef) {
        this.http = http$$1;
        this.data = data;
        this.dialogRef = dialogRef;
        this.baseApiUrl = 'http://localhost';
        this.headers = new Headers({ 'Content-Type': 'application/json' });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.options = new RequestOptions({ headers: this.headers });
        this.inputFields = this.data.displayed_columns;
        console.log('data arrived!');
        console.log(this.inputFields);
        this.model_name = this.data.model_name;
        if (this.inputFields && this.inputFields[0] != '') {
            this.window_title = 'Edit ' + this.model_name;
        }
        else {
            this.window_title = 'New ' + this.model_name;
        }
    }
    /**
     * @return {?}
     */
    addEditObject() {
        if (this.inputFields && this.inputFields[0].field_value) {
            const /** @type {?} */ requestData = [];
            for (let /** @type {?} */ i = 0; i < this.inputFields.length; i++) {
                if (this.inputFields[i]) {
                    requestData[this.inputFields[i].field_name] = this.inputFields[i].field_value;
                }
            }
            this.http.get(this.baseApiUrl).subscribe(result => {
                if (result) {
                    console.log(this.model_name + ' modified!');
                    this.dialogRef.close('success');
                }
                else {
                    this.dialogRef.close('error');
                }
            });
        }
        else {
            console.log('adding object..');
            const /** @type {?} */ requestData = [];
            for (let /** @type {?} */ i = 0; i < this.inputFields.length; i++) {
                requestData[this.inputFields[i].field_name] = this.inputFields[i].field_value;
            }
            this.addObject(this.model_name, Object.assign({}, requestData)).map(data => data.json()).subscribe(result => {
                if (result) {
                    this.dialogRef.close(result);
                }
                else {
                    this.dialogRef.close(result);
                }
            });
        }
    }
    /**
     * @param {?} obj
     * @param {?} data
     * @return {?}
     */
    addObject(obj, data) {
        let /** @type {?} */ add_object_url = this.baseApiUrl + '/api/' + obj + 's/';
        if (obj == 'category') {
            add_object_url = this.baseApiUrl + '/api/categories/';
        }
        return this.http.post(add_object_url, data, this.options);
    }
    /**
     * @param {?} obj
     * @param {?} object_id
     * @return {?}
     */
    deleteObject(obj, object_id) {
        let /** @type {?} */ delete_object_url = this.baseApiUrl + '/api/' + obj + 's/' + object_id + '/';
        if (obj == 'category') {
            delete_object_url = this.baseApiUrl + '/api/categories/' + object_id + '/';
        }
        const /** @type {?} */ response = this.http.delete(delete_object_url, this.options)
            .map(res => res.json());
        console.log('Object ' + object_id + ' deleted!');
        return response;
    }
    /**
     * @param {?} obj
     * @param {?} data
     * @return {?}
     */
    patchObject(obj, data) {
        console.log('patching....');
        let /** @type {?} */ patch_object_url = this.baseApiUrl + '/api/' + obj + 's/' + data[obj + '_id'] + '/';
        if (obj == 'category') {
            patch_object_url = this.baseApiUrl + '/api/categories/' + data[obj + '_id'] + '/';
        }
        return this.http.patch(patch_object_url, Object.assign({}, data), this.options)
            .map(res => res.json());
    }
}
ObjectAddDeleteModifyComponent.decorators = [
    { type: Component, args: [{
                selector: 'app-object-add-delete-modify',
                template: `


    <h2 mat-dialog-title>
        {{window_title}}
    </h2>

    <mat-form-field class="fullwidth"  *ngFor="let field of inputFields">
        <input *ngIf="field.type !='textarea'" matInput class="fullwidth"
               [(ngModel)]="field.field_value"
               [id]="field.field_name"
               [disabled]="field.read_only == true"
               placeholder="{{field.label}}"
               [type]="field.type">
        <textarea matInput *ngIf="field.type == 'textarea'"
                  [(ngModel)]="field.field_value"
                  [id]="field.field_name"
                  [disabled]="field.read_only == true"
                  placeholder="{{field.label}}">
        </textarea>
    </mat-form-field>

    <div class="flex-space-end">
        <div>
            <button  mat-raised-button  *ngIf="show_warning != true" (click)="addEditObject()">
                <i class="material-icons">add</i>
                Save
            </button>
            <button  mat-raised-button color="warn" *ngIf="show_warning != true &&  data != null" (click)="show_warning = true">
                <i class="material-icons">delete</i>
                Delete
            </button>
        </div>
        <div *ngIf="show_warning == true">
            <mat-dialog-actions  class="flex-space-between flex-center-vertical">
                <span class="small-margin-horizontal">Are you sure?</span>
                <button mat-button mat-dialog-close (click)="show_warning = false" >No</button>
                <button mat-button (click)="deleteObject($event.target.value)" mat-dialog-close>Yes</button>
            </mat-dialog-actions>
        </div>
    </div>
  `,
                styles: [`

  `]
            },] },
];
/**
 * @nocollapse
 */
ObjectAddDeleteModifyComponent.ctorParameters = () => [
    { type: Http, },
    { type: undefined, decorators: [{ type: Inject, args: [MAT_DIALOG_DATA,] },] },
    { type: MatDialogRef, },
];

var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
class ObjectCrudComponent {
    /**
     * @param {?} http
     * @param {?} dialog
     * @param {?} snackBar
     */
    constructor(http$$1, dialog, snackBar) {
        this.http = http$$1;
        this.dialog = dialog;
        this.snackBar = snackBar;
        this.headers = new Headers({ 'Content-Type': 'application/json' });
        this.search_field_changed = new EventEmitter();
        this.columns_def = [];
        this.options = new RequestOptions({ headers: this.headers });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.field = false;
        this.getObjectFeed();
        this.reloadColumnVisibility();
    }
    /**
     * @return {?}
     */
    getObjectFeed() {
        return __awaiter(this, void 0, void 0, function* () {
            this.length = this.getCount(this.model_name);
            this.dataSource = new ApiDataSource(this.http, this.paginator, this.sorter, this.model_name, this.baseApiUrl, this.urlSegments, this.search_field_changed, this.options);
        });
    }
    /**
     * @return {?}
     */
    reloadColumnVisibility() {
        for (let /** @type {?} */ i = 0; i < this.displayed_columns.length; i++) {
            if (this.displayed_columns[i].visible == true) {
                this.columns_def.push(this.displayed_columns[i].field_name);
            }
        }
    }
    /**
     * @param {?} colName
     * @return {?}
     */
    changeColumnVisibility(colName) {
        this.columns_def = [];
        for (const /** @type {?} */ i of this.displayed_columns) {
            if (i.field_name == colName) {
                i.visible = !i.visible;
                console.log('changing column ' + colName + ' visibility to ' + i.visible);
            }
        }
        this.reloadColumnVisibility();
    }
    /**
     * @param {?} element
     * @return {?}
     */
    addEditObject(element) {
        const /** @type {?} */ inputFields = [];
        if (element) {
            for (let /** @type {?} */ i = 0; i < this.displayed_columns.length; i++) {
                inputFields.push(this.displayed_columns[i]);
                inputFields[i]['field_value'] = element[this.displayed_columns[i].field_name];
            }
        }
        else {
            for (let /** @type {?} */ i = 0; i < this.displayed_columns.length; i++) {
                inputFields.push(this.displayed_columns[i]);
                inputFields[i]['field_value'] = '';
            }
        }
        const /** @type {?} */ dialogRef = this.dialog.open(ObjectAddDeleteModifyComponent, {
            width: '50%',
            data: {
                displayed_columns: inputFields,
                model_name: this.model_name
            }
        });
        dialogRef.afterClosed().subscribe(result => {
            if (result) {
                this.snackBar.open(result, 'OK', {
                    duration: 4000
                });
                this.getObjectFeed();
            }
        });
    }
    /**
     * @param {?} value
     * @return {?}
     */
    searchFieldChanged(value) {
        this.search_field_changed.emit(value);
    }
    /**
     * @param {?} obj
     * @return {?}
     */
    getCount(obj) {
        // TODO add optional where options
        if (obj === 'category') {
            obj = 'categories';
        }
        else
            obj += 's';
        const /** @type {?} */ response = this.http.get(this.baseApiUrl + '/' + obj + '/count')
            .map(res => res.json());
        return response;
    }
}
ObjectCrudComponent.decorators = [
    { type: Component, args: [{
                selector: 'app-object-crud',
                template: `
      <mat-card-title class="text-underlined">
          Manage {{model_name}}
      </mat-card-title>
      <mat-card-content>
          <p *ngIf="show_warning">
              {{window_message}}
          </p>
          <form class="find-object-form">
              <div class="flex-space-between flex-center-vertical small-margin-vertical">
                  <div>
                      <mat-form-field >
                          <input matInput #filter ([ngModel])="search_keyword" (change)="searchFieldChanged(filter.value)" placeholder="Find {{model_name}}">
                      </mat-form-field>
                      <mat-form-field class="small-margin-horizontal">
                          <mat-select multiple placeholder="Hide columns">
                              <mat-option selected  *ngFor="let column of displayed_columns" (click)="changeColumnVisibility(column.field_name)">
                                  {{column.label}}
                              </mat-option>
                          </mat-select>
                      </mat-form-field>
                  </div>
                  <div>
                      <button  mat-raised-button  (click)="addEditObject()"><i class="material-icons">add</i>New {{model_name}}</button>
                  </div>
              </div>
          </form>
          <mat-table #table [dataSource]="dataSource" matSort>
                  <ng-container *ngFor="let column of displayed_columns;"  matColumnDef="{{column.field_name}}" >
                          <mat-header-cell mat-sort-header  *matHeaderCellDef> {{column.label}} </mat-header-cell>
                          <mat-cell *matCellDef="let element"> {{element[column.field_name]}} </mat-cell>
                  </ng-container>
              <mat-header-row  *matHeaderRowDef="columns_def"></mat-header-row>
              <mat-row (click)="addEditObject(element)" *matRowDef="let row; let element; columns: columns_def;"></mat-row>
          </mat-table>
          <mat-paginator #paginator
                         [length]="length | async"
                         [pageSize]="25"
                         [pageSizeOptions]="[5, 10, 25, 100]">
          </mat-paginator>
      </mat-card-content>
    `,
                styles: [`

    `]
            },] },
];
/**
 * @nocollapse
 */
ObjectCrudComponent.ctorParameters = () => [
    { type: Http, },
    { type: MatDialog, },
    { type: MatSnackBar, },
];
ObjectCrudComponent.propDecorators = {
    'paginator': [{ type: ViewChild, args: [MatPaginator,] },],
    'sorter': [{ type: ViewChild, args: [MatSort,] },],
    'displayed_columns': [{ type: Input },],
    'labels': [{ type: Input },],
    'model_name': [{ type: Input },],
    'baseApiUrl': [{ type: Input },],
    'urlSegments': [{ type: Input },],
    'search_field_changed': [{ type: Output },],
};
class ApiDataSource extends DataSource {
    /**
     * @param {?} http
     * @param {?} paginator
     * @param {?} sorter
     * @param {?} model_name
     * @param {?} baseApiUrl
     * @param {?} urlSegments
     * @param {?} search_field_changed
     * @param {?} options
     */
    constructor(http$$1, paginator, sorter, model_name, baseApiUrl, urlSegments, search_field_changed, options) {
        super();
        this.http = http$$1;
        this.paginator = paginator;
        this.sorter = sorter;
        this.model_name = model_name;
        this.baseApiUrl = baseApiUrl;
        this.urlSegments = urlSegments;
        this.search_field_changed = search_field_changed;
        this.options = options;
        this.dataLoad = new BehaviorSubject$1([]);
        this.sort_direction = 'asc';
        this.order_by = '';
        this.order_by = this.model_name + '_id';
    }
    /**
     * @param {?} obj
     * @param {?} category
     * @param {?} keyword
     * @param {?} limit
     * @param {?} selected_page
     * @param {?} sort
     * @param {?} order_by
     * @return {?}
     */
    filterFeed(obj, category, keyword, limit, selected_page, sort, order_by) {
        let /** @type {?} */ response;
        var /** @type {?} */ filter_feed_url = this.baseApiUrl + this.urlSegments['afterBaseUrl'] + obj + 's/?' + this.urlSegments['selectedPage'] + selected_page + '&' + this.urlSegments['limit'] + limit;
        if (keyword != undefined && keyword != '') {
            filter_feed_url += '&' + this.urlSegments['filterKeyword'] + keyword;
        }
        if (order_by !== '') {
            filter_feed_url += '&' + this.urlSegments['order'] + order_by;
        }
        else {
            filter_feed_url += this.urlSegments['order'];
        }
        if (sort !== '' && sort != undefined) {
            filter_feed_url += '&' + this.urlSegments['sort'] + sort;
        }
        else {
            filter_feed_url += '&' + this.urlSegments['sort'] + obj + '_id';
        }
        console.log(filter_feed_url);
        response = this.http.get(filter_feed_url)
            .map(res => res.json());
        return response;
    }
    /**
     * @return {?}
     */
    connect() {
        this.http.get(this.baseApiUrl + '/' + this.model_name + 's/').subscribe({
            next: (value) => {
                this.dataLoad.next(value.json());
            }
        });
        //TODO adjust to catch paginator change
        this.paginator.page.subscribe(() => {
            this.filterFeed(this.model_name, [], '', this.paginator.pageSize, this.paginator.pageIndex, this.sorter.active, this.sort_direction).subscribe({
                next: (value) => {
                    this.dataLoad.next(value);
                }
            });
        });
        //TODO adjust to reload on search field change
        this.search_field_changed.subscribe((val) => {
            this.http.get(this.baseApiUrl).subscribe({
                next: (value) => {
                    this.dataLoad.next(value);
                }
            });
        });
        //TODO adjust to change sorting direction
        this.sorter.sortChange.subscribe(() => {
            if (this.sort_direction == 'asc') {
                this.sort_direction = 'desc';
            }
            else {
                this.sort_direction = 'asc';
            }
            this.filterFeed(this.model_name, [], '', this.paginator.pageSize, this.paginator.pageIndex, this.sorter.active, this.sort_direction).subscribe({
                next: (value) => {
                    this.dataLoad.next(value);
                }
            });
        });
        return Observable$1.merge(this.dataLoad);
    }
    /**
     * @return {?}
     */
    disconnect() {
    }
}

class ObjectCrudModule {
}
ObjectCrudModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    HttpModule,
                    FormsModule,
                    BrowserAnimationsModule,
                    MatDatepickerModule, MatDialogModule, MatMenuModule, MatButtonModule, MatCardModule, MatTabsModule,
                    MatTableModule, MatPaginatorModule, MatInputModule, MatSnackBarModule, MatSortModule, MatCheckboxModule,
                    MatSelectModule, MatFormFieldModule
                ],
                declarations: [
                    ObjectAddDeleteModifyComponent,
                    ObjectCrudComponent
                ],
                exports: [
                    ObjectCrudComponent,
                    ObjectAddDeleteModifyComponent
                ],
                entryComponents: [
                    ObjectAddDeleteModifyComponent
                ]
            },] },
];
/**
 * @nocollapse
 */
ObjectCrudModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { ObjectCrudModule, ObjectAddDeleteModifyComponent as ɵa, ObjectCrudComponent as ɵb };
export { ɵf, ɵg, ɵd, ɵe, ɵc, BrowserAnimationsModule, NoopAnimationsModule, ɵBrowserAnimationBuilder, ɵBrowserAnimationFactory, ɵAnimationRenderer, ɵAnimationRendererFactory } from '@angular/platform-browser/animations';
//# sourceMappingURL=angular-4-crud-module.js.map
